/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
package com.jbtravel.beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Stateful;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaQuery;

import com.jbtravel.domain.Fee;
import com.jbtravel.domain.Flight;
import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.ReservationStatusType;
import com.jbtravel.domain.Seat;
import com.jbtravel.domain.User;
import com.jbtravel.domain.ValMealtype;
import com.jbtravel.rules.RulesEngine;
import com.jbtravel.view.FlightOptions;

@Named
@Stateful
public class ReservationBean implements Serializable {

	private static final long serialVersionUID = 1L;
	@EJB
	private UserBean userBean;

	@PersistenceContext
	private EntityManager entityManager;
	
	@Inject
	private RulesEngine rulesEngine;

	public Reservation find(Integer reservationId) {
		return entityManager.find(Reservation.class, reservationId);
	}

	public Reservation create(String username, Flight flight,
			ValMealtype meal, FlightOptions options) {
		Reservation reservation = new Reservation();
		
		User user = userBean.findUser(username);
		if (user == null)
			throw new IllegalArgumentException("User not found.");
		reservation.setUser(user);
		
		reservation.setFlight(flight);
		reservation.setValMealtype(meal);
		reservation.setCheckedin(0);

		reservation.setBagcheck(options.isCheckBags());
		reservation.setBags(options.getNumBags());
		reservation.setTripinsurance(options.isSelectTripInsurance());
		reservation.setUpgradeclass(options.isUpgradeClass());
		reservation.setUpgradecomfort(options.isSelectComfortPackage());
		reservation.setUpgradedrink(options.isSelectDrinkPackage());
		reservation.setUpgradeentertainment(options
				.isSelectEntertainmentPackage());
		reservation.setUpgrademeal(options.isUpgradeMeal());
		reservation.setStatus(ReservationStatusType.PENDING);

		entityManager.persist(reservation);

		// calculate fees
		
		for (Object o : rulesEngine.fire(reservation, user)) {
			if (o instanceof Fee) {
				Fee fee = (Fee) o;
				fee.setReservation(reservation);
				entityManager.persist(fee);
				reservation.addFee(fee);
			}
		}

		user.getReservations().add(reservation);
		return reservation;
	}

	public List<Reservation> getAll(String username) {

		User user = userBean.findUser(username);
		if (user != null) {
			List<Reservation> reservations = new ArrayList<Reservation>();
			if (user.getReservations() != null) {
				for (Reservation reservation : user.getReservations()) {
					reservations.add(reservation);
				}
			}
			return reservations;
		}
		return null;
	}

	public List<Reservation> getAll() {

		CriteriaQuery<Reservation> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(Reservation.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(Reservation.class)))
				.getResultList();
	}
	
	public void makeSeatSelection(Reservation reservation, Seat seat) {
		Reservation _reservation = entityManager.merge(reservation);
		Seat _seat = entityManager.merge(seat);
		_reservation.setSeat(_seat);
		_seat.getReservations().add(_reservation);
	}

}