/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
package com.jbtravel.beans;

import java.io.Serializable;
import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaQuery;

import com.jbtravel.domain.Flight;
import com.jbtravel.domain.Seat;

@Named
@Stateless
public class SeatBean implements Serializable {

	private static final long serialVersionUID = 1L;

	@PersistenceContext
	private EntityManager entityManager;

	public List<Seat> getAll() {

		CriteriaQuery<Seat> criteria = this.entityManager.getCriteriaBuilder()
				.createQuery(Seat.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(Seat.class))).getResultList();
	}
	
	public List<Seat> getAllAvailable(Flight flight) {
		@SuppressWarnings("unchecked")
		List<Seat> seats = (List<Seat>) entityManager
				.createQuery(
						"select s from Flight f join f.plane p join p.seats s where f = :flight" +
				" and s not in (select r.seat from Reservation r where r.flight = :flight)"
						)
				.setParameter("flight", flight).getResultList();
		return seats;
	}

	public Seat findSeat(Integer id) {
		return entityManager.find(Seat.class, id);
	}

}