package com.jbtravel.domain;

import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 * The persistent class for the flight database table.
 * 
 */
@Entity
@Table(schema = "JBTRAVEL")
public class Flight implements Serializable {
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Integer flightid;

	private Timestamp arrivaldate;

	private Timestamp departuredate;

	private BigDecimal firstclassprice;

	private Integer flightnumber;

	private BigDecimal price;

	private Integer seatstaken;

	// bi-directional many-to-one association to Plane
	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "planeid")
	private Plane plane;

	// bi-directional many-to-one association to Route
	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "routeid")
	private Route route;

	// bi-directional many-to-one association to Reservation
	@OneToMany(mappedBy = "flight")
	private Set<Reservation> reservations;

	public Flight() {
	}

	public Integer getFlightid() {
		return this.flightid;
	}

	public void setFlightid(Integer flightid) {
		this.flightid = flightid;
	}

	public Timestamp getArrivaldate() {
		return this.arrivaldate;
	}

	public void setArrivaldate(Timestamp arrivaldate) {
		this.arrivaldate = arrivaldate;
	}

	public Timestamp getDeparturedate() {
		return this.departuredate;
	}

	public void setDeparturedate(Timestamp departuredate) {
		this.departuredate = departuredate;
	}

	public BigDecimal getFirstclassprice() {
		return this.firstclassprice;
	}

	public void setFirstclassprice(BigDecimal firstclassprice) {
		this.firstclassprice = firstclassprice;
	}

	public Integer getFlightnumber() {
		return this.flightnumber;
	}

	public void setFlightnumber(Integer flightnumber) {
		this.flightnumber = flightnumber;
	}

	public BigDecimal getPrice() {
		return this.price;
	}

	public void setPrice(BigDecimal price) {
		this.price = price;
	}

	public Integer getSeatstaken() {
		return this.seatstaken;
	}

	public void setSeatstaken(Integer seatstaken) {
		this.seatstaken = seatstaken;
	}

	public Plane getPlane() {
		return this.plane;
	}

	public void setPlane(Plane plane) {
		this.plane = plane;
	}

	public Route getRoute() {
		return this.route;
	}

	public void setRoute(Route route) {
		this.route = route;
	}

	public Set<Reservation> getReservations() {
		return this.reservations;
	}

	public void setReservations(Set<Reservation> reservations) {
		this.reservations = reservations;
	}

}