package com.jbtravel.rules;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;

import org.kie.api.KieServices;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.Seat;
import com.jbtravel.domain.User;

@Singleton
@Startup
public class SeatRulesEngine  implements Serializable {

	private static final long serialVersionUID = 1L;

	private KieContainer container; 

	public List<Seat> fire(List<Seat> availableSeats, Reservation reservation, User customer) {
		
		KieSession ksession = container.newKieSession("SeatSession");
		if (ksession == null)
			throw new IllegalStateException("KIE SeatSession was not built");
		
		for(Object seat : availableSeats) {
			ksession.insert(seat);
		}
		ksession.insert(reservation);
		ksession.insert(customer);
	
		ksession.fireAllRules();
		
		List<Seat> seats = new ArrayList<Seat>();
		for (Object o : ksession.getObjects())
			if (o instanceof Seat)
			  seats.add((Seat) o);
		
		ksession.dispose();
		
		return seats;
	}
	
	@PostConstruct
	public void build(){
		KieServices kieServices = KieServices.Factory.get();
		container = kieServices.newKieClasspathContainer();
		if(RuleUtils.checkErrors(container)){
			throw new IllegalArgumentException("There are rule syntax errors");
		}

	}
	
}
