/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

package com.jbtravel.view;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;

import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.inject.Inject;
import javax.inject.Named;

import com.jbtravel.beans.ReservationBean;
import com.jbtravel.beans.SeatBean;
import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.Seat;
import com.jbtravel.rules.SeatRulesEngine;

@Named
@ConversationScoped
public class Confirmation implements Serializable {

	private static final long serialVersionUID = 1L;
	private Integer reservationId;
	private Reservation reservation;
	private Seat seat;

	@Inject
	private ReservationBean reservationBean;
	@Inject
	private SeatBean seatBean;

	@Inject
	Conversation conversation;
	
	@Inject
	SeatRulesEngine rulesEngine;

	public void recall() {
		this.reservation = reservationBean.find(reservationId);
	}

	public Integer getReservationId() {
		return reservationId;
	}

	public void setReservationId(Integer reservationId) {
		this.reservationId = reservationId;
	}

	public Reservation getReservation() {
		return reservation;
	}

	public void setReservation(Reservation reservation) {
		this.reservation = reservation;
	}

	public String showSeatSelections() {
		if (conversation.isTransient())
			conversation.begin();
		reservation = reservationBean.find(reservationId);
		return "jbtFSelectSeat";
	}

	public List<Seat> getSeats() {
		// use rule engine to adjust seats that are available and allowed
		List<Seat> availableSeats = seatBean.getAllAvailable(reservation.getFlight());
		List<Seat> allowedSeats = rulesEngine.fire(availableSeats, reservation, reservation.getUser());
		Collections.sort(allowedSeats);
		return allowedSeats;
	}

	public String selectSeat() {
		reservationBean.makeSeatSelection(reservation, seat);
		conversation.end();
		return "jbtFSearchConfirm?faces-redirect=true&resId="
				+ reservation.getReservationid();
	}

	public Seat getSeat() {
		return this.seat;
	}

	public void setSeat(Seat seat) {
		this.seat = seat;
	}

}
