/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

package com.jbtravel.view;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;

import com.jbtravel.beans.AirportBean;
import com.jbtravel.beans.FlightBean;
import com.jbtravel.beans.RouteBean;
import com.jbtravel.domain.Airport;
import com.jbtravel.domain.Flight;
import com.jbtravel.domain.Route;

@Named
@ConversationScoped
public class FlightSearch implements Serializable {

	private static final long serialVersionUID = 1L;
	private Date departureDate;
	private String originAirport;
	private String destAirport;
	private List<Flight> flights;
	@Inject
	AirportBean airportBean;
	@Inject
	RouteBean routeBean;
	@Inject
	FlightBean flightBean;

	@Inject
	Conversation conversation;

	Flight flight;

	public String search() {

		if (conversation.isTransient())
			conversation.begin();

		Airport oa = airportBean.findAirport(originAirport);
		Airport da = airportBean.findAirport(destAirport);
		Route route = routeBean.findRoute(oa, da);
		if (route != null) {
			Flight flightCriteria = new Flight();
			flightCriteria.setRoute(route);
			if (departureDate != null)
				flightCriteria.setDeparturedate(new Timestamp(departureDate
					.getTime()));
			this.flights = flightBean.search(flightCriteria);
		} else {
			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(String.format("No routes for %s/%s",
							originAirport, destAirport), String.format(
							"There are no airlines that fly between %s and %s",
							oa.getAirportname(), da.getAirportname())));
			return "jbtFSearch";
		}

		return "jbtFSearchDisplayFlights?faces-redirect=true";
	}

	public Date getDepartureDate() {
		return departureDate;
	}

	public void setDepartureDate(Date departureDate) {
		this.departureDate = departureDate;
	}

	public String getOriginAirport() {
		return originAirport;
	}

	public void setOriginAirport(String originAirport) {
		this.originAirport = originAirport;
	}

	public String getDestAirport() {
		return destAirport;
	}

	public void setDestAirport(String destAirport) {
		this.destAirport = destAirport;
	}

	public List<Flight> getFlights() {
		return flights;
	}

	public Flight getFlight() {
		return flight;
	}

	public void setFlight(Flight flight) {
		this.flight = flight;
	}
}
