/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

package com.jbtravel.view;

import java.io.Serializable;

import javax.enterprise.context.SessionScoped;
import javax.enterprise.inject.Produces;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;

import com.jbtravel.beans.UserBean;
import com.jbtravel.domain.User;

@SessionScoped
@Named
public class Identity implements Serializable {

	private static final long serialVersionUID = 1L;
	private boolean loggedIn = false;
	private boolean reservationInProgress = false;
	private User user = new User();

	@Inject
	UserBean userBean;

	@Named
	@Produces
	Credentials credentials = new Credentials();

	public String login() {

		User user = userBean.findUser(credentials.getUsername());
		if (user == null) {
			FacesContext
					.getCurrentInstance()
					.addMessage(
							null,
							new FacesMessage(
									FacesMessage.SEVERITY_ERROR,
									"Invalid user name",
									String.format(
											"User name, %s, does not exist in the database.  Create a new account?",
											credentials.getUsername())));
			return null;
		}

		if (!user.getPassword().equals(credentials.getPassword())) {
			FacesContext
					.getCurrentInstance()
					.addMessage(
							null,
							new FacesMessage(
									FacesMessage.SEVERITY_ERROR,
									"Invalid password",
									String.format(
											"Password for User name, %s, does not match the one on the account.",
											user.getUsername())));
			return null;
		}

		this.user = user;
		this.loggedIn = true;

		if (this.reservationInProgress)
			return "jbtFSearchReserve?faces-redirect=true";

		return "jbtHome?faces-redirect=true";
	}

	public String logout() {
		this.loggedIn = false;
		this.reservationInProgress = false;
		this.user = new User();
		if (!credentials.isRememberMe())
			this.credentials = new Credentials();
		else
			this.credentials.setPassword(null);

		return "jbtHome";
	}

	public String create() {
		this.user = userBean.createUser(this.user);
		this.loggedIn = true;
		return "jbtHome?faces-redirect=true";
	}

	public String update() {
		userBean.updateUser(this.user);
		return "jbtHome?faces-redirect=true";
	}
	
	public void refresh() {
		this.user = userBean.findUser(this.user.getUsername());
	}

	public boolean isLoggedIn() {
		return this.loggedIn;
	}

	public boolean isReservationInProgress() {
		return this.reservationInProgress;
	}

	public void setReservationInProgress(boolean reservationInProgress) {
		this.reservationInProgress = reservationInProgress;
	}

	public User getUser() {
		return this.user;
	}

	public Credentials getCredentials() {
		return credentials;
	}

}
