/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

package com.jbtravel.view;

import java.io.Serializable;

import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.inject.Named;

import com.jbtravel.beans.FlightBean;
import com.jbtravel.beans.ReservationBean;
import com.jbtravel.domain.Flight;
import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.ValMealtype;

@Named("reservation")
@ConversationScoped
public class ReservationTransaction implements Serializable {

	private static final long serialVersionUID = 1L;

	private Flight flight;

	private ValMealtype meal;
	private Reservation reservation;

	@Inject
	private Identity identity;
	@Inject
	private Conversation conversation;
	@Inject
	private FlightBean flightBean;
	@Inject
	private ReservationBean reservationBean;

	
	@Named("options")
	@Produces
	private FlightOptions options = new FlightOptions();

	public String selectFlight(Integer flightId) {

		this.flight = flightBean.getFlight(flightId);

		if (!identity.isLoggedIn()) {
			identity.setReservationInProgress(true);
			return "jbtLogin";
		}

		return "jbtFSearchReserve?faces-redirect=true";
	}

	public String confirm() {
		this.reservation = reservationBean.create(identity.getUser()
				.getUsername(), flight, meal, options);
		conversation.end();
		identity.refresh();
		return "jbtFSearchConfirm?faces-redirect=true&resId="
				+ reservation.getReservationid();
	}

	public Flight getFlight() {
		return flight;
	}

	public void setFlight(Flight flight) {
		this.flight = flight;
	}

	public ValMealtype getMeal() {
		return meal;
	}

	public void setMeal(ValMealtype meal) {
		this.meal = meal;
	}

	public Reservation getReservation() {
		return reservation;
	}

}
