/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

package com.jbtravel.beans;

import java.math.BigDecimal;
import java.util.List;

import javax.inject.Inject;

import org.jboss.arquillian.junit.Arquillian;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.jbtravel.BaseWebTestTemplate;
import com.jbtravel.domain.Airport;
import com.jbtravel.domain.DrinkFee;
import com.jbtravel.domain.Fee;
import com.jbtravel.domain.Flight;
import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.ReservationStatusType;
import com.jbtravel.domain.Route;
import com.jbtravel.domain.Seat;
import com.jbtravel.domain.User;
import com.jbtravel.domain.ValMealtype;
import com.jbtravel.view.FlightOptions;

@RunWith(Arquillian.class)
public class ReservationBeanTest extends BaseWebTestTemplate {

	@Inject
	ReservationBean reservationBean;
	@Inject
	AirportBean airportBean;
	@Inject
	RouteBean routeBean;
	@Inject
	FlightBean flightBean;
	@Inject
	SeatBean seatBean;
	@Inject
	MealBean mealBean;
	@Inject
	UserBean userBean;

	@Test
	public void testCreateAndFind() {
		Airport oa = airportBean.findAirport("SFO");
		Assert.assertNotNull("Finding SFO airport", oa);
		Airport da = airportBean.findAirport("JFK");
		Assert.assertNotNull("Finding JFK airport", da);
		Route route = routeBean.findRoute(oa, da);
		Assert.assertNotNull("Routes not found", route);
		Flight flightCriteria = new Flight();
		flightCriteria.setRoute(route);
		List<Flight> flights = flightBean.search(flightCriteria);
		Assert.assertNotNull(flights);
		Assert.assertTrue("flights found", flights.size() > 0);
		List<ValMealtype> meals = mealBean.getAll();
		Assert.assertNotNull(meals);
		Assert.assertTrue("meals found", meals.size() > 0);
		List<Seat> seats = seatBean.getAll();
		Assert.assertNotNull(seats);
		Assert.assertTrue("seats found", seats.size() > 0);

		Flight flight = flights.get(0);
		ValMealtype meal = meals.get(0);

		User user = userBean.findUser("admin");
		int miles = user.getMiles();

		FlightOptions options = new FlightOptions();
		options.setSelectDrinkPackage(true);
		Reservation reservation = reservationBean.create("admin", flight, meal,
				options);
		Assert.assertNotNull(reservation);
		Assert.assertNotNull(reservation.getReservationid());
		Assert.assertTrue("Reservation ID is valid",
				reservation.getReservationid() != 0);
		Assert.assertEquals(1, reservation.getFees().size());
		Fee fee = reservation.getFees().iterator().next();
		Assert.assertNotNull(fee);
		Assert.assertTrue(fee instanceof DrinkFee);
		DrinkFee dfee = (DrinkFee) fee;
		Assert.assertEquals(new BigDecimal("10.00"), dfee.getAmount());

		Reservation foundReservation = reservationBean.find(reservation
				.getReservationid());
		Assert.assertNotNull(foundReservation);
		Assert.assertEquals(reservation.getReservationid(),
				foundReservation.getReservationid());
		Assert.assertEquals(1, foundReservation.getFees().size());
		Assert.assertEquals("Gold", foundReservation.getUser()
				.getLoyaltylevel());
		Assert.assertEquals(ReservationStatusType.CONFIRMED,
				foundReservation.getStatus());
		User _user = userBean.findUser("admin");
		Assert.assertEquals(
				(Integer) (miles + flight.getRoute().getDistance()),
				_user.getMiles());

	}

}
