/*
 Copyright (c) 2012 Red Hat, Inc.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

   1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above
   copyright notice, this list of conditions and the following
   disclaimer in the documentation and/or other materials provided
   with the distribution.

 THIS SOFTWARE IS PROVIDED BY RED HAT, INC. ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL RED HAT, INC. BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

package com.jbtravel.domain;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import org.jboss.arquillian.junit.Arquillian;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.jbtravel.BaseWebTestTemplate;

@RunWith(Arquillian.class)
public class EntityTest extends BaseWebTestTemplate {

	@PersistenceContext
	EntityManager em;

	@Test
	public void testAirports() {
		CriteriaBuilder builder = this.em.getCriteriaBuilder();
		CriteriaQuery<Airport> criteria = builder.createQuery(Airport.class);
		Root<Airport> root = criteria.from(Airport.class);
		TypedQuery<Airport> query = this.em.createQuery(criteria.select(root)
				.where(builder.equal(root.<String> get("airportcode"), "SFO")));

		try {
			Airport airport = query.getSingleResult();
			Assert.assertNotNull(airport);
			Assert.assertEquals("SFO", airport.getAirportcode());
		} catch (NoResultException e) {
			Assert.fail("Could not find SFO airport.");
		}
	}

	@Test
	public void testFlights() {
		CriteriaBuilder builder = this.em.getCriteriaBuilder();
		CriteriaQuery<Flight> criteria = builder.createQuery(Flight.class);
		Root<Flight> root = criteria.from(Flight.class);
		TypedQuery<Flight> query = this.em.createQuery(criteria.select(root));

		List<Flight> flights = query.getResultList();
		Assert.assertNotNull(flights);
		Assert.assertTrue(flights.size() > 0);
		Flight flight = flights.get(0);
		Assert.assertEquals(new BigDecimal("907.55"), flight.getPrice());
	}

	@Test
	public void testPlanes() {
		CriteriaBuilder builder = this.em.getCriteriaBuilder();
		CriteriaQuery<Plane> criteria = builder.createQuery(Plane.class);
		Root<Plane> root = criteria.from(Plane.class);
		TypedQuery<Plane> query = this.em.createQuery(criteria.select(root));

		List<Plane> planes = query.getResultList();
		Assert.assertNotNull(planes);
		Assert.assertTrue(planes.size() > 0);

	}

	@Test
	public void testReservations() {
		CriteriaBuilder builder = this.em.getCriteriaBuilder();
		CriteriaQuery<Reservation> criteria = builder
				.createQuery(Reservation.class);
		Root<Reservation> root = criteria.from(Reservation.class);
		TypedQuery<Reservation> query = this.em.createQuery(criteria
				.select(root));

		List<Reservation> reservations = query.getResultList();
		Assert.assertNotNull(reservations);

	}

	@Test
	public void testRoutes() {
		CriteriaBuilder builder = this.em.getCriteriaBuilder();
		CriteriaQuery<Route> criteria = builder.createQuery(Route.class);
		Root<Route> root = criteria.from(Route.class);
		TypedQuery<Route> query = this.em.createQuery(criteria
				.select(root)
				.where(builder.equal(root.<Airport> get("airport1")
						.<Integer> get("airportid"), 71))
				.where(builder.equal(root.<Airport> get("airport2")
						.<Integer> get("airportid"), 34)));

		List<Route> routes = query.getResultList();
		Assert.assertNotNull(routes);
		Assert.assertTrue("Could not find any JFK-SFO routes.",
				routes.size() > 0);

	}

	@Test
	public void testSeats() {
		CriteriaBuilder builder = this.em.getCriteriaBuilder();
		CriteriaQuery<Seat> criteria = builder.createQuery(Seat.class);
		Root<Seat> root = criteria.from(Seat.class);
		TypedQuery<Seat> query = this.em.createQuery(criteria.select(root));

		List<Seat> seats = query.getResultList();
		Assert.assertNotNull(seats);
		Assert.assertTrue(seats.size() > 0);

	}

	@Test
	public void testUsers() {
		User user = em.find(User.class, 21);
		Assert.assertNotNull(user);
		Assert.assertEquals("admin", user.getFirstname());
		Assert.assertEquals("admin", user.getLastname());
		Assert.assertEquals("admin admin", user.toString());
	}

	@Test
	public void testMeals() {
		ValMealtype meal = em.find(ValMealtype.class, "Kosher");
		Assert.assertNotNull(meal);
	}

	@Test
	public void testSeatClasses() {
		ValSeatclass seat = em.find(ValSeatclass.class, "First");
		Assert.assertNotNull(seat);
	}

	@Test
	public void testSeatTypes() {
		ValSeattype seatType = em.find(ValSeattype.class, "Aisle");
		Assert.assertNotNull(seatType);
	}

}
