package com.jbtravel.rules;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.Seat;
import com.jbtravel.domain.User;
import com.jbtravel.domain.ValSeatclass;

public class SeatRulesTest extends BaseRulesTestTemplate {

	public SeatRulesTest() {
		super("SeatSession");
	}
	
	@Test
	public void testFirstClass() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(true);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("None");
		user.setAge(20);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals(1, seats.size());	
		Seat seat = seats.get(0);
		Assert.assertEquals("1A",seat.getSeatname());
	}
	
	@Test
	public void testGoldUnderAge() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(false);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("Gold");
		user.setAge(20);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals("Did not find number of seats", 3, seats.size());	
		int found = 0;
		for (Seat seat : seats) {
			if ("20A".equals(seat.getSeatname()))
				++found;
			if ("11A".equals(seat.getSeatname()))
				++found;
			if ("4A".equals(seat.getSeatname()))
				++found;
		}
		Assert.assertEquals("Did not find expected seats", 3,found);
	}
	
	@Test
	public void testGoldOfAge() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(false);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("Gold");
		user.setAge(21);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals("Did not find number of seats", 4, seats.size());	
		int found = 0;
		for (Seat seat : seats) {
			if ("20A".equals(seat.getSeatname()))
				++found;
			if ("12A".equals(seat.getSeatname()))
				++found;
			if ("11A".equals(seat.getSeatname()))
				++found;
			if ("4A".equals(seat.getSeatname()))
				++found;
		}
		Assert.assertEquals("Did not find expected seats", 4,found);
	}
	
	@Test
	public void testSilver() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(false);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("Silver");
		user.setAge(21);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals("Did not find number of seats", 3, seats.size());	
		int found = 0;
		for (Seat seat : seats) {
			if ("20A".equals(seat.getSeatname()))
				++found;
			if ("11A".equals(seat.getSeatname()))
				++found;
			if ("4A".equals(seat.getSeatname()))
				++found;
		}
		Assert.assertEquals("Did not find expected seats", 3,found);
	}
	
	@Test
	public void testSilverWithUpgrade() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(true);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("Silver");
		user.setAge(21);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals(1, seats.size());	
		Seat seat = seats.get(0);
		Assert.assertEquals("1A",seat.getSeatname());
	}
	
	@Test
	public void testBronze() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(false);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("Bronze");
		user.setAge(21);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals("Did not find number of seats", 2, seats.size());	
		int found = 0;
		for (Seat seat : seats) {
			if ("20A".equals(seat.getSeatname()))
				++found;
			if ("11A".equals(seat.getSeatname()))
				++found;
		}
		Assert.assertEquals("Did not find expected seats", 2,found);
	}
	
	@Test
	public void testNFF() {
		createSeats();
		Reservation reservation = new Reservation();
		reservation.setUpgradeclass(false);
		ksession.insert(reservation);
		User user = new User();
		user.setLoyaltylevel("None");
		user.setAge(21);
		ksession.insert(user);
		
		ksession.fireAllRules();
		
		List<Seat> seats = getAllowedSeats();
		Assert.assertEquals("Did not find number of seats", 1, seats.size());	
		int found = 0;
		for (Seat seat : seats) {
			if ("20A".equals(seat.getSeatname()))
				++found;
		}
		Assert.assertEquals("Did not find expected seats", 1,found);
	}
	
	private void createSeats() {
		ValSeatclass seatClassFirst = new ValSeatclass();
		seatClassFirst.setClass_("First");
		ValSeatclass seatClassEc1 = new ValSeatclass();
		seatClassEc1.setClass_("Economy 1");	
		ValSeatclass seatClassEc2 = new ValSeatclass();
		seatClassEc2.setClass_("Economy 2");
		ValSeatclass seatClassEc3 = new ValSeatclass();
		seatClassEc3.setClass_("Economy 3");
		Seat seat1A = new Seat();
		seat1A.setValSeatclass(seatClassFirst);
		seat1A.setSeatname("1A");
		ksession.insert(seat1A);
		Seat seat4A = new Seat();
		seat4A.setValSeatclass(seatClassEc1);
		seat4A.setSeatname("4A");
		ksession.insert(seat4A);
		Seat seat11A = new Seat();
		seat11A.setValSeatclass(seatClassEc2);
		seat11A.setSeatname("11A");
		ksession.insert(seat11A);
		Seat seat12A = new Seat();
		seat12A.setValSeatclass(seatClassEc2);
		seat12A.setExitrow(true);
		seat12A.setSeatname("12A");
		ksession.insert(seat12A);
		Seat seat20A = new Seat();
		seat20A.setValSeatclass(seatClassEc3);
		seat20A.setSeatname("20A");
		ksession.insert(seat20A);
	}
	
	private List<Seat> getAllowedSeats() {
		List<Seat> seats = new ArrayList<Seat>();
		
		for(Object o : ksession.getObjects()) {
			if (o instanceof Seat)
				seats.add((Seat) o);		
		}
		
		return seats;
	}
}
