package com.jbtravel.rules;

import static org.junit.Assert.assertEquals;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.rule.FactHandle;

import com.jbtravel.domain.DrinkFee;
import com.jbtravel.domain.Fee;
import com.jbtravel.domain.Reservation;
import com.jbtravel.domain.User;

public class UpgradeRulesTest extends BaseRulesTestTemplate {

	
	@Test
	public void drinkUpgradeWithFeeTest() {
		Reservation reservation = new Reservation();
		reservation.setUpgradedrink(true);
		User user = new User();
		user.setLoyaltylevel("Gold");
		user.setAge(21);

		ksession.insert(reservation);
		FactHandle userHandle = ksession.insert(user);
		ksession.fireAllRules();

		List<Fee> fees = extractFees(ksession);
		assertEquals(1, fees.size());
		assertFee(new DrinkFee(new BigDecimal("10.00")), fees.get(0));

		ksession.delete(userHandle);
		user.setLoyaltylevel("Silver");
		userHandle = ksession.insert(user);
		ksession.fireAllRules();

		fees = extractFees(ksession);
		assertEquals(1, fees.size());
		assertFee(new DrinkFee(new BigDecimal("15.00")), fees.get(0));

	}
	
	@Test
	public void drinkUpgradeWithNoFeeTest() {
		Reservation reservation = new Reservation();
		reservation.setUpgradedrink(false);
		User user = new User();
		user.setLoyaltylevel("Gold");
		user.setAge(88);

		ksession.insert(reservation);
		ksession.insert(user);
		ksession.fireAllRules();

		List<Fee> fees = extractFees(ksession);
		assertEquals(0, fees.size());
	}
	
	@Test
	public void invalidDrinkUpgradeTest() {
		Reservation reservation = new Reservation();
		reservation.setUpgradedrink(true);
		User user = new User();
		user.setLoyaltylevel("Gold");
		user.setAge(20);
		
		FactHandle reservationFact = ksession.insert(reservation);
		FactHandle userFact = ksession.insert(user);
		ksession.fireAllRules();

		List<Fee> fees = extractFees(ksession);
		assertEquals(0, fees.size());
		Reservation afReservation = (Reservation) ksession.getObject(reservationFact);
		assertEquals(false, afReservation.isUpgradedrink());
		
		ksession.delete(userFact);
		ksession.delete(reservationFact);
		reservation.setUpgradedrink(true);
		user.setAge(44);
		user.setLoyaltylevel("None");
		
		userFact = ksession.insert(user);
		reservationFact = ksession.insert(reservation);
		ksession.fireAllRules();
		fees = extractFees(ksession);
		assertEquals(0, fees.size());
		afReservation = (Reservation) ksession.getObject(reservationFact);
		assertEquals(false, afReservation.isUpgradedrink());
		
		ksession.delete(userFact);
		ksession.delete(reservationFact);
		reservation.setUpgradedrink(true);
		user.setLoyaltylevel("Bronze");
		
		userFact = ksession.insert(user);
		reservationFact = ksession.insert(reservation);
		ksession.fireAllRules();
		fees = extractFees(ksession);
		assertEquals(0, fees.size());
		afReservation = (Reservation) ksession.getObject(reservationFact);
		assertEquals(false, afReservation.isUpgradedrink());
		
	}

	private List<Fee> extractFees(KieSession ksession) {
		List<Fee> fees = new ArrayList<Fee>();
		for (Object o : ksession.getObjects()) {
			if (o instanceof Fee)
				fees.add((Fee) o);
		}
		return fees;
	}

	private void assertFee(Fee expectedFee, Fee actualFee) {

		assertEquals("Expected " + expectedFee.getClass().getSimpleName(),
				expectedFee.getClass(), actualFee.getClass());

		assertEquals(expectedFee.getAmount(), actualFee.getAmount());
	}
}
